#include "error.hh"

template<typename T,unsigned Ndims>
BAS_DYNARRAY<T,Ndims>::BAS_DYNARRAY(empty_struct e, ...)
    : BAS_MAGICARRAY_ImplSTATIC<T>(0, Ndims)
{
    va_list ap;
    va_start(ap, e);
    BAS_MAGICARRAY_ImplSTATIC<T>::InitDims(ap);
    va_end(ap);
    RedimUpdate();
}

template<typename T,unsigned Ndims>
T& BAS_DYNARRAY<T,Ndims>::Index(unsigned numdims, ...)
{
    va_list ap;
    va_start(ap,numdims);
    T& res = DoIndex(numdims, ap);
    va_end(ap);
    return res;
}

template<typename T,unsigned Ndims>
T& BAS_DYNARRAY<T,Ndims>::DoIndex(unsigned numdims, va_list ap)
{
    return BAS_MAGICARRAY_ImplSTATIC<T>::Index(numdims, ap);
}

template<typename T,unsigned Ndims>
void BAS_DYNARRAY<T,Ndims>::Redim(unsigned numdims, ...)
{
    va_list ap;
    va_start(ap,numdims);
    DoRedim(numdims,ap);
    va_end(ap);
}

template<typename T,unsigned Ndims>
void BAS_DYNARRAY<T,Ndims>::DoRedim(unsigned numdims, va_list ap)
{
    if(numdims != Ndims) { B_ERROR(9); return; }
    BAS_MAGICARRAY_ImplSTATIC<T>::InitDims(ap);
    RedimUpdate();
}

template<typename T,unsigned Ndims>
void BAS_DYNARRAY<T,Ndims>::RedimUpdate()
{
    container.clear(); // first delete all contents
    container.resize(
        BAS_MAGICARRAY_ImplSTATIC<T>::
        Volume()); // then resize it to required size
    BAS_MAGICARRAY_ImplSTATIC<T>::
    data = &container[0]; // then update data pointer
}

template<typename T,unsigned Ndims>
void BAS_DYNARRAY<T,Ndims>::Erase()
{
    for(unsigned a=0; a<Ndims; ++a)
    {
        BAS_MAGICARRAY_ImplSTATIC<T>::dims[a].first = 0;
        BAS_MAGICARRAY_ImplSTATIC<T>::dims[a].second = 0;
    }
    RedimUpdate();
}
